@@ -40,6 +40,7 @@
 #include <cerrno>
 #include <string>
 #include <linux/cec-funcs.h>
+#include "cec-htng-funcs.h"
 
 #ifdef ANDROID
 #include <android-config.h>
@@ -47,6 +48,8 @@
 #include <config.h>
 #endif
 
+#include <cec-common.h>
+
 #define ARRAY_SIZE(a) \
 	(sizeof(a) / sizeof(*a))
 
